@@ -97,6 +97,8 @@ class Recipe:
                     ):
                     # Something has changed. Blow away the instance.
                     self.install()
+                elif options.get('site-zcml'):
+                    self.build_package_includes()
 
             # Nothing has changed.
             return location
@@ -154,6 +156,16 @@ class Recipe:
             if port_base:
                 port_base = 'port-base %s' % port_base
             http_address = options.get('http-address', '8080')
+            ftp_address = options.get('ftp-address', '')
+            if ftp_address:
+                ftp_address = ftp_server_template % ftp_address
+            webdav_address = options.get('webdav-address', '')
+            if webdav_address:
+                webdav_conn_close = options.get(
+                                        'webdav-force-connection-close',
+                                        'off')
+                webdav_address = webdav_server_template % (webdav_address, 
+                                                           webdav_conn_close)
             effective_user = options.get('effective-user', '')
             if effective_user:
                effective_user = 'effective-user %s' % effective_user 
@@ -200,7 +212,7 @@ class Recipe:
                 default_zpublisher_encoding = 'default-zpublisher-encoding %s' % default_zpublisher_encoding
 
             relstorage = options.get('rel-storage')
-            if relstorage is not None:
+            if relstorage:
                 def _split(el):
                     el = el.split(None, 1)
                     return len(el) == 2 and el or None
@@ -261,15 +273,28 @@ class Recipe:
                     zeo_client_name = 'zeo-client-name %s' % zeo_client_name
                 if zeo_client_client:
                     zeo_client_client = 'client %s' % zeo_client_client
+                if options.get('zeo-username', ''):
+                    if not options.get('zeo-password', ''):
+                        raise zc.buildout.UserError('No ZEO password specified')
+
+                    zeo_authentication = zeo_authentication_template % dict(
+                            realm = options.get('zeo-realm', 'ZEO'),
+                            username = options.get('zeo-username'),
+                            password = options.get('zeo-password'))
+                else:
+                    zeo_authentication = ''
+
                 if blob_storage:
                     storage_snippet_template = zeo_blob_storage_template
                 else:
                     storage_snippet_template = zeo_storage_template
+
                 storage_snippet = storage_snippet_template % dict(
                     blob_storage = blob_storage,
                     shared_blob_dir = shared_blob_dir,
                     zeo_address = zeo_address,
                     zeo_client_cache_size = zeo_client_cache_size,
+                    zeo_authentication = zeo_authentication,
                     zeo_client_client = zeo_client_client,
                     zeo_storage = zeo_storage,
                     zeo_var_dir=zeo_var_dir,
@@ -312,6 +337,8 @@ class Recipe:
                                         storage_snippet = storage_snippet.strip(),
                                         port_base = port_base,
                                         http_address = http_address,
+                                        ftp_address = ftp_address,
+                                        webdav_address = webdav_address,
                                         zserver_threads = zserver_threads,
                                         zodb_cache_size = zodb_cache_size,
                                         zeo_client_name = zeo_client_name,
@@ -443,10 +470,15 @@ if __name__ == '__main__':
         """Create ZCML slugs in etc/package-includes
         """
         location = self.options['location']
+        sitezcml_path = os.path.join(location, 'etc', 'site.zcml')
         zcml = self.options.get('zcml')
+        site_zcml = self.options.get('site-zcml')
+
+        if site_zcml:
+            open(sitezcml_path, 'w').write(site_zcml)
+            return
 
         if zcml:
-            sitezcml_path = os.path.join(location, 'etc', 'site.zcml')
             if not os.path.exists(sitezcml_path):
                 # Zope 2.9 does not have a site.zcml so we copy the
                 # one out from Five.
@@ -527,6 +559,12 @@ blob_storage_template="""
     </blobstorage>
 """
 
+zeo_authentication_template="""
+    realm %(realm)s
+      username %(username)s
+      password %(password)s
+""".strip()
+
 zeo_storage_template="""
     # ZEOStorage database
     <zeoclient>
@@ -535,6 +573,7 @@ zeo_storage_template="""
       name zeostorage
       var %(zeo_var_dir)s
       cache-size %(zeo_client_cache_size)s
+      %(zeo_authentication)s
       %(zeo_client_client)s
     </zeoclient>
 """.strip()
@@ -549,6 +588,7 @@ zeo_blob_storage_template="""
       name zeostorage
       var %(zeo_var_dir)s
       cache-size %(zeo_client_cache_size)s
+      %(zeo_authentication)s
       %(zeo_client_client)s
     </zeoclient>
 """.strip()
@@ -567,6 +607,21 @@ access_event_logfile = """
   </logfile>
 """.strip()
 
+ftp_server_template = """
+<ftp-server>
+  # valid key is "address"
+  address %s
+</ftp-server>
+"""
+
+webdav_server_template = """
+<webdav-source-server>
+  # valid keys are "address" and "force-connection-close"
+  address %s
+  force-connection-close %s
+</webdav-source-server>
+"""
+
 # The template used to build zope.conf
 zope_conf_template="""\
 instancehome %(instance_home)s
@@ -601,6 +656,9 @@ verbose-security %(verbose_security)s
   # use-wsgi on
 </http-server>
 
+%(ftp_address)s
+%(webdav_address)s
+
 <zodb_db main>
     # Main database
     cache-size %(zodb_cache_size)s
